@@ -11,7 +11,7 @@
 /* Message sent to the application by library. `conn' associates the
    message to a specific connection.  `conn', however, may be NULL.
    The `type' indicates the type of the message sent by the library.
-   The applicationi can for example filter the message according the
+   The application can for example filter the message according the
    type. */
 
 static void
@@ -23,8 +23,10 @@ silc_say(SilcClient client, SilcClientConnection conn,
 
 
 /* Message for a channel. The `sender' is the sender of the message
-   The `channel' is the channel. The `msg' is the message.  Note that
-   `msg' maybe NULL. */
+   The `channel' is the channel. The `message' is the message.  Note
+   that `message' maybe NULL.  The `flags' indicates message flags
+   and it is used to determine how the message can be interpreted
+   (like it may tell the message is multimedia message). */
 
 static void
 silc_channel_message(SilcClient client, SilcClientConnection conn,
@@ -37,7 +39,10 @@ silc_channel_message(SilcClient client, SilcClientConnection conn,
 
 
 /* Private message to the client. The `sender' is the sender of the
-   message. */
+   message. The message is `message'and maybe NULL.  The `flags'  
+   indicates message flags  and it is used to determine how the message
+   can be interpreted (like it may tell the message is multimedia
+   message). */
 
 static void
 silc_private_message(SilcClient client, SilcClientConnection conn,
@@ -117,8 +122,8 @@ silc_command_reply(SilcClient client, SilcClientConnection conn,
    silc_client_close_connection. */
 
 static void
-silc_connect(SilcClient client, SilcClientConnection conn,
-	     SilcClientConnectionStatus status)
+silc_connected(SilcClient client, SilcClientConnection conn,
+	       SilcClientConnectionStatus status)
 {
 
 }
@@ -130,8 +135,8 @@ silc_connect(SilcClient client, SilcClientConnection conn,
    received from server. */
 
 static void
-silc_disconnect(SilcClient client, SilcClientConnection conn,
-		SilcStatus status, const char *message)
+silc_disconnected(SilcClient client, SilcClientConnection conn,
+		  SilcStatus status, const char *message)
 {
 
 }
@@ -154,8 +159,9 @@ silc_get_auth_method(SilcClient client, SilcClientConnection conn,
 
 /* Verifies received public key. The `conn_type' indicates which entity
    (server, client etc.) has sent the public key. If user decides to trust
-   the key may be saved as trusted public key for later use. The
-   `completion' must be called after the public key has been verified. */
+   the application may save the key as trusted public key for later
+   use. The `completion' must be called after the public key has been
+   verified. */
 
 static void
 silc_verify_public_key(SilcClient client, SilcClientConnection conn,
@@ -204,7 +210,7 @@ silc_failure(SilcClient client, SilcClientConnection conn,
    silc_client_perform_key_agreement). If TRUE is returned also the
    `completion' and `context' arguments must be set by the application. */
 
-static int
+static bool
 silc_key_agreement(SilcClient client, SilcClientConnection conn,
 		   SilcClientEntry client_entry, const char *hostname,
 		   SilcUInt16 port, SilcKeyAgreementCallback *completion,
@@ -264,8 +270,8 @@ SilcClientOperations ops = {
   silc_notify,
   silc_command,
   silc_command_reply,
-  silc_connect,
-  silc_disconnect,
+  silc_connected,
+  silc_disconnected,
   silc_get_auth_method,
   silc_verify_public_key,
   silc_ask_passphrase,
